import { PointInt2D } from "../../Utils/Math.utils";
import { getWorldChunkAt, worldCoordinatesToChunkLocalCoordinates } from "../../WorldGeneration/WorldGen";
import { TerrainTile } from "../../WorldGeneration/WorldObjects/TerrainTile/TerrainTile";

export class NavigationGridTile
{
    position = new PointInt2D();
    movementCost = 0;
    isObstacle = false;
    /**
     * @type {TerrainTile}
     */
    optionalTerrainTileRef = null;

    /**
     * 
     * @param {PointInt2D} pos 
     * @param {Number} cost 
     * @param {Boolean} isObstacle 
     */
    constructor(pos, cost, isObstacle, ref = null)
    {
        this.position = pos;
        this.movementCost = cost;
        this.isObstacle = isObstacle;
        this.optionalTerrainTileRef = ref;
    }
}

/**
 * @param {NavigationGridTile} tile 
 */
function NavigationGridTileSettingsChangeFunctor(tile) {
    
}

export class NavigationGridChunk
{
    /**
     * @type {Map<String, NavigationGridTile>}
     */
    _gridTiles = new Map();

    /**
     * 
     * @param {NavigationGridTile} element 
     * @param {String} id 
     */
    addToChunk(element, id)
    {
        this._gridTiles.set(id, element);
    }

    getFromChunkById(id)
    {
        return this._gridTiles.get(id);
    }

    getFromChunk(xWorld, yWorld)
    {
        let coords = worldCoordinatesToChunkLocalCoordinates(xWorld, yWorld);
        return this._gridTiles.get(coords.x+"_"+coords.y);
    }

    /**
     * 
     * @param {String} id 
     * @param {NavigationGridTileSettingsChangeFunctor} func 
     */
    changeNavigationGridTileSettings(id, func)
    {
        let t = this._gridTiles.get(id);
        if(t)
        {
            func(t);
            return true;
        }
        return false;
    }
}


export class NavigationGrid
{   

    /**
     * 
     * @param {Number} xWorld 
     * @param {Number} yWorld 
     */
    static getNavigationGridTile(xWorld, yWorld)
    {
        let chunk = getWorldChunkAt(xWorld, yWorld);
        if(chunk)
        {
            return chunk.navigationGridChunk.getFromChunk(xWorld, yWorld);
        }
        return undefined;
    }
}

export function getNavigationGridTile(xWorld, yWorld)
{
    let chunk = getWorldChunkAt(xWorld, yWorld);
    if(chunk)
    {
        return chunk.navigationGridChunk.getFromChunk(xWorld, yWorld);
    }
    return undefined;
}